#define PYGGIMISC_MODULE
#include "pyggi.h"
#include <ggi/misc.h>

#define GGIMISC_FUNC(s, c) PyCFunction_New(&_ggimisc_methods[c], (PyObject*)s)

static pyggi_C_api * ggi_c_api;


static PyObject*
ggimisc_waitRayPos(Visual *self, PyObject * args) {
	int x,y;	
	
	if(!PyArg_ParseTuple(args, "ii",&x,&y))
		return NULL;
	if(!ggiWaitRayPos(self->vis, &x, &y)) {
		PyErr_SetString(PyExc_SystemError,"ggiWaitRayPos failed");
		return NULL;
	}
	return Py_BuildValue("(ii)",x,y);
}

enum {
	MISC_WAITRAYPOS
};

static struct PyMethodDef _ggimisc_methods[] = {
	{ "waitRayPos", (PyCFunction)ggimisc_waitRayPos,   METH_VARARGS,
	  (char*)NULL  },
	{ NULL, NULL, NULL, NULL }
};


static int
ggimisc_setattr(Visual *self , char * name, PyObject * v) {
	switch(name[0]) {
	case 's':
		if(!strcmp(name,"splitline")) {
			if (!PyInt_Check(v)) {
				PyErr_SetString(
					PyExc_TypeError, "integer required");
				return -1;
			}
			if(!ggiSetSplitline(self->vis, PyInt_AsLong(v))) {
				PyErr_SetString(PyExc_SystemError,
						"ggiSetSplitline failed");
				return -1;
			}
			return 1;
		}
		break;
	}
	return 0;
}

static PyObject* ggimisc_getattr(Visual *self, char * name) {
	int x,y;
	
	switch(name[0]) {
	case 'w':
		if(!strcmp(name,"waitRayPos"))
			return GGIMISC_FUNC(self, MISC_WAITRAYPOS);
		break;
	case 'r':
		if(!strcmp(name,"rayPos")) {
			if(!ggiGetRayPos(self->vis, &x, &y)) {
				PyErr_SetString(PyExc_SystemError,
						"ggiGetRayPos failed");
				return NULL;
			}
			return Py_BuildValue("(ii)",x,y);
		}
		break;
	}
	return NULL;
}

static PyObject*
ggimisc_attach(PyObject*self, PyObject*args) {
	Visual   * o;
	
	if(!PyArg_ParseTuple(args, "O", (PyObject**)(&o))) {
		return NULL;
	}
	ASSERT_VISUAL(o, NULL);

	if(ggiMiscInit()<0) {
		PyErr_SetString(PyExc_SystemError, "ggiMiscInit failed");
		return NULL;
	}
	
	if(ggiMiscAttach(o->vis)<0) {
		PyErr_SetString(PyExc_SystemError, "ggiMiscAttach failed");
		ggiMiscExit();
		return NULL;
	}
	
	if(ggi_c_api->attach(o, "ggimisc",
			     &ggimisc_getattr, &ggimisc_setattr) < -1) {
		PyErr_SetString(PyExc_SystemError,
				"pyggi_attachExtension failed");
		ggiMiscDetach(o->vis);
		ggiMiscExit();
		return NULL;
	}
	
	Py_INCREF(Py_None);
	return Py_None;
}

static PyObject*
ggimisc_detach(PyObject*self, PyObject*args) {
	Visual   * o;
	
	if(!PyArg_ParseTuple(args, "O", (PyObject**)(&o))) return NULL;
	ASSERT_VISUAL(o, NULL);
	
	if(ggi_c_api->detach(o, "ggimisc") < -1) {
		PyErr_SetString(PyExc_SystemError,
				"pyggi_detachExtension failed");
		return NULL;
	}
	
	if(ggiMiscDetach(o->vis) < 0) {
		PyErr_SetString(PyExc_SystemError, "ggiMiscDetach failed");
		return NULL;
	}
	if(ggiMiscExit() < 0) {
		PyErr_SetString(PyExc_SystemError, "ggiMiscExit failed");
		return NULL;
	}
	
	Py_INCREF(Py_None);
	return Py_None;
}

static struct PyMethodDef ggimisc_methods[] = {
	{ "attach",  (PyCFunction)ggimisc_attach, METH_VARARGS, (char*)NULL },
	{ "detach",  (PyCFunction)ggimisc_detach, METH_VARARGS, (char*)NULL },
	{ NULL, NULL, NULL, NULL }
};



static void importModules(void) {
	PyObject *ggi;
	PyObject *ggidict;
	PyObject *capiobj;
	
	ggi= PyImport_ImportModule("GGI._ggimodule");
	if (ggi!= NULL) {
		ggidict = PyModule_GetDict(ggi);
		capiobj = PyDict_GetItemString(ggidict, "_ggimodule_C_api");
		if (PyCObject_Check(capiobj)) {
			ggi_c_api=(pyggi_C_api *)PyCObject_AsVoidPtr(capiobj);
		}
		/* 
		   Py_DECREF(ggidict);
		*/
	} else {
		Py_FatalError("Can't import GGI._ggimodule");
		return;
	}
}

#ifndef PyMODINIT_FUNC
#define PyMODINIT_FUNC void
#endif
PyMODINIT_FUNC
init_ggimiscmodule(void) {
	importModules();
	Py_InitModule3("_ggimiscmodule", ggimisc_methods, (char *)NULL);
}
